local math = require("math")

-- Standard awesome library
local gears = require("gears")
local awful = require("awful")
require("awful.autofocus")
-- Widget and layout library
local wibox = require("wibox")
-- Theme handling library
local beautiful = require("beautiful")
-- Notification library
local naughty = require("naughty")
local menubar = require("menubar")
local hotkeys_popup = require("awful.hotkeys_popup").widget
-- Enable hotkeys help widget for VIM and other apps
-- when client with a matching name is opened:
require("awful.hotkeys_popup.keys")

-- {{{ Error handling
-- Check if awesome encountered an error during startup and fell back to
-- another config (This code will only ever execute for the fallback config)
if awesome.startup_errors then
    naughty.notify({ preset = naughty.config.presets.critical,
                     title = "Oops, there were errors during startup!",
                     text = awesome.startup_errors })
end

-- Handle runtime errors after startup
do
    local in_error = false
    awesome.connect_signal("debug::error", function (err)
        -- Make sure we don't go into an endless error loop
        if in_error then return end
        in_error = true

        naughty.notify({ preset = naughty.config.presets.critical,
                         title = "Oops, an error happened!",
                         text = tostring(err) })
        in_error = false
    end)
end
-- }}}

-- {{{ Variable definitions
-- Themes define colours, icons, font and wallpapers.
-- beautiful.init("/home/syrup/.config/awesome/theme.lua")
beautiful.init("/home/syrup/.config/awesome/themes/macos-dark-c/theme.lua")

-- This is used later as the default terminal and editor to run.
terminal = os.getenv("TERMINAL") or "xterm"
editor = os.getenv("EDITOR") or "nano"
editor_cmd = terminal .. " -e " .. editor

-- ENVIRONMENT VARIABLES
local server_ip = io.open("/home/syrup/.server_ip"):read("*l")

-- Default modkey.
-- Usually, Mod4 is the key with a logo between Control and Alt.
-- If you do not like this or do not have such a key,
-- I suggest you to remap Mod4 to another key using xmodmap or other tools.
-- However, you can use another modifier like Mod1, but it may interact with others.
modkey = "Mod4"

-- Table of layouts to cover with awful.layout.inc, order matters.
awful.layout.layouts = {
    awful.layout.suit.tile,
    awful.layout.suit.tile.left,
    -- awful.layout.suit.tile.bottom,
    -- awful.layout.suit.tile.top,
    awful.layout.suit.fair,
    awful.layout.suit.fair.horizontal,
    awful.layout.suit.spiral,
    awful.layout.suit.spiral.dwindle,
    awful.layout.suit.max,
    -- awful.layout.suit.max.fullscreen,
    awful.layout.suit.magnifier,
    awful.layout.suit.corner.nw,
    -- awful.layout.suit.corner.ne,
    awful.layout.suit.corner.sw,
    -- awful.layout.suit.corner.se,
    awful.layout.suit.floating,
}
-- }}}

-- {{{ Helper functions
local function client_menu_toggle_fn()
    local instance = nil

    return function ()
        if instance and instance.wibox.visible then
            instance:hide()
            instance = nil
        else
            instance = awful.menu.clients({ theme = { width = 250 } })
        end
    end
end
-- }}}

-- {{{ Menu
-- Create a launcher widget and a main menu
myawesomemenu = {
   { "hotkeys", function() return false, hotkeys_popup.show_help end},
   { "manual", terminal .. " -e man awesome" },
   { "edit config", editor_cmd .. " " .. awesome.conffile },
   { "restart", awesome.restart },
   { "quit", function() awesome.quit() end}
}

mymainmenu = awful.menu({ items = { { "awesome", myawesomemenu, beautiful.awesome_icon },
                                    { "open terminal", terminal }
                                  }
                        })

mylauncher = awful.widget.launcher({ image = beautiful.awesome_icon,
                                     menu = mymainmenu })

-- Menubar configuration
menubar.utils.terminal = terminal -- Set the terminal for applications that require it
-- }}}

-- {{{ Wibar
-- Create a textclock widget
mytextclock = wibox.widget.textclock()

-- Custom area

local myupdatestatus = wibox.widget {
  {
    id           = "mytu",
    text         = " 🗘 ",
    align        = "left",
    widget       = wibox.widget.textbox,
  },
  layout      = wibox.layout.stack,
  set_updates = function(self, update_a, update_b)
    local text_buf  = ""
    if update_a > 0 then
      text_buf = text_buf .. "  " .. update_a .. " "
    end
    if update_b > 0 then
      text_buf = text_buf .. "  " .. update_b .. " "
    end
    if update_a == 0 and update_b == 0 then
      text_buf = " ✔ "
    end
    self.mytu.text = text_buf
  end,
}

function update_updatestatus()
  local f = io.popen("checkupdates | wc -l")
  local pacup = f:read("n")
  -- local f = io.popen("pacaur -k | wc -l")
  -- local aurup = f:read("n")
  local aurup = 0
  myupdatestatus:set_updates(pacup, aurup)
end

-- update_updatestatus() -- Make async

--  gears.timer {
--    timeout   = 3600,
--    autostart = true,
--    callback  = update_updatestatus,
--  }


-- DS4 battery status
local mydsbattery = wibox.widget {
  {
    min_value    = 0,
    max_value    = 60,
    value        = 0,
    forced_width = 50,
    color = "#ffffff",
    background_color = "#222266",
    id           = "mydspb",
    widget       = wibox.widget.progressbar,
  },
  {
    id           = "mydstb",
    text         = "---%",
    align        = "center",
    widget       = wibox.widget.textbox,
  },
  layout      = wibox.layout.stack,
  set_battery = function(self, val)
    if val == 0 then
      self.visible = false
    else
      self.visible = true
    end
    self.mydstb.text  = math.ceil(val / 60 * 100) .."%"
    self.mydspb.value = val
    if val > 50 then
      self.mydspb.color = "#226622"
    elseif val > 25 then
      self.mydspb.color = "#666622"
    else
      self.mydspb.color = "#662222"
    end
  end
}

function update_dsbattery()
  local f = io.open("/sys/class/power_supply/sony_controller_battery_f4:93:9f:a6:1f:4d/capacity")
  if f == nil then
    mydsbattery:set_battery(0)
    return
  end
  local cap = f:read("*n")
  -- local f = io.open("/sys/class/power_supply/sony_controller_battery_f4:93:9f:a6:1f:4d/status")
  -- local stat = f:read("*s")

  mydsbattery:set_battery(cap)
end


local mybatterybar = wibox.widget {
  {
    min_value    = 0,
    max_value    = 100,
    value        = 0,
    forced_width = 50,
    color = "#ffffff",
    background_color = "#555555",
    id           = "mypb",
    widget       = wibox.widget.progressbar,
  },
  {
    id           = "mytb",
    text         = "---%",
    align        = "center",
    widget       = wibox.widget.textbox,
  },
  layout      = wibox.layout.stack,
  set_battery = function(self, val)
    self.mytb.text  = val.."%"
    self.mypb.value = val
    if val > 50 then
      self.mypb.color = "#228822"
    elseif val > 25 then
      self.mypb.color = "#888822"
    else
      self.mypb.color = "#882222"
    end
  end,
}

function update_batterystatus()
  awful.spawn.easy_async("cat /sys/class/power_supply/BAT0/capacity /sys/class/power_supply/ACAD/online", function(sout, serr, exitr, exitc)
    local f = sout:gmatch("[^\n]+")
    local res = tonumber(f(), 10)
    local charging = f() == "1"
    if (not charging) and (res <= 5) then
      awful.spawn{"mpv", "--no-vid", "/home/syrup/.local/share/resources/alert24.mp3"}
    end
    mybatterybar:set_battery(res)
  end)
end

local function human_readble_size(s, base, delta)
  local suffixes = {"B", "KiB", "MiB", "GiB", "TiB", "EiB"}
  local idx = base
  if idx == nil then
    idx = 1
  end
  while math.abs(s) >= 1024 and idx < #suffixes do
    s = s / 1024
    idx = idx + 1
  end
  if delta then
    return string.format("%+g%s", s, suffixes[idx])
  else
    return string.format("%g%s", s, suffixes[idx])
  end
end

local filesystem_last_usage = {}

local myfilesystem = wibox.widget {
  {
    id           = "mydf",
    text         = "nil ",
    align        = "left",
    widget       = wibox.widget.textbox,
  },
  layout      = wibox.layout.stack,
  update_df = function(self, data_array)
    self.mydf.text = ""
    for i = 1, #data_array, 3 do
      local name = data_array[i+2]
      local last = filesystem_last_usage[name]
      if last == nil then
	last = 0
      end
      local change = data_array[i] - last
      filesystem_last_usage[name] = data_array[i]

      self.mydf.text = self.mydf.text .. "[" ..
	name .. ": " .. data_array[i+1]
      if change ~= 0 then
	self.mydf.text = self.mydf.text .. " " .. human_readble_size(change, 2, true)
      end
      self.mydf.text = self.mydf.text .. "] "
    end
  end
}

function update_filesystem()
  local f = io.popen("df")
  local arr = {}
  for line in f:lines() do
    if string.find(line, "/$") ~= nil or string.find(line, "/home$") ~= nil then
      local pos = 1
      for v in string.gmatch(line, "[^%s]+") do
	if pos == 3 or pos >= 5 then
	  local n = tonumber(v)
	  if n ~= nil then
	    arr[#arr + 1] = n
	  else
	    arr[#arr + 1] = v
	  end
	end
	pos = pos + 1
      end
    end
  end
  myfilesystem:update_df(arr)
end

local folder_last_usage = {}
local folders = {"~/.ipfs"}

local myfolders = wibox.widget {
  {
    id           = "mydu",
    text         = "nil ",
    align        = "left",
    widget       = wibox.widget.textbox,
  },
  layout      = wibox.layout.stack,
  update_du = function(self, data_array)
    self.mydu.text = ""
    for k, v in pairs(data_array) do
      local last = folder_last_usage[k]
      if last == nil then
	last = 0
      end
      local change = data_array[k] - last
      folder_last_usage[k] = data_array[k]

      self.mydu.text = self.mydu.text .. "[" ..
	k .. ": " .. human_readble_size(data_array[k], 2)
      if change ~= 0 then
	self.mydu.text = self.mydu.text .. " " .. human_readble_size(change, 2, true)
      end
      self.mydu.text = self.mydu.text .. "] "
    end
  end
}

function update_folders()
  local arr = {}
  for _, v in ipairs(folders) do
    print(v)
    local f = io.popen("du -s " .. v)
    arr[v] = f:read("*n")
  end
  myfolders:update_du(arr)
end

local function bool_to_text(b)
  if b then
    return "OK"
  else
    return "FAIL"
  end
end

local myvpn = wibox.widget {
  {
    {
      id           = "myvp",
      text         = " Unknown! ",
      align        = "center",
      widget       = wibox.widget.textbox,
    },
    id = "myvpc",
    widget = wibox.container.background,
    bg = "#662222",
  },
  layout      = wibox.layout.stack,
  update_vpn = function(self, status_tunnel, status_server)
    if status_tunnel then
      self.myvpc.bg = "#226622"
    else
      if status_server then
        self.myvpc.bg = "#666622"
      else
        self.myvpc.bg = "#662222"
      end
    end
    self.myvpc.myvp.text = " T:" .. bool_to_text(status_tunnel) .. " S:" .. bool_to_text(status_server) .. " "
  end
}

function update_vpn()
  awful.spawn.easy_async_with_shell("ping -c 1 -W 1 10.11.0.1 > /dev/null", function(sout, serr, exitr, exitc)
    if exitc ~= 0 then
      awful.spawn.easy_async_with_shell("ping -c 1 -W 1 " .. server_ip .. " > /dev/null", function(sout2, serr2, exitr2, exitc2)
        myvpn:update_vpn(false, exitc2 == 0)
      end)
    else
      -- Tunnel implies server, we assume that it is statistically to unlikely
      -- to encounter a server on this IP in a local network
      myvpn:update_vpn(true, true)
    end
  end)
end

local mywifi = wibox.widget {
  {
    {
      id           = "mywf",
      text         = "null",
      align        = "center",
      widget       = wibox.widget.textbox,
    },
    id = "mywfc",
    widget = wibox.container.background,
    bg = "#662222",
  },
  layout      = wibox.layout.stack,
  update_wifi = function(self, status, ap)
    if status == "COMPLETED" then
      self.mywfc.bg = "#226622"
    else
      self.mywfc.bg = "#662222"
    end
    if ap ~= "" then
      self.mywfc.mywf.text = " " .. ap .. " "
    else
      self.mywfc.mywf.text = " " .. status .. " "
    end
  end
}

function update_wifi()
  awful.spawn.easy_async_with_shell("wpa_cli status | grep ^wpa_state= | cut -d \"=\" -f 2", function(sout, sin, exitr, exitc)
    awful.spawn.easy_async_with_shell("wpa_cli status | grep ^ssid= | cut -d \"=\" -f 2", function(sout2, sin2, exitr2, exitc2)
      mywifi:update_wifi(sout:gsub("\n", ""), sout2:gsub("\n", ""))
    end)
  end)
end

local mysystemload = wibox.widget {
  {
    {
    id = "mylg",
    min_value = 0,
    max_value = 5,
    forced_width = 40,
    step_width = 2,
    step_spacing = 0,
    color = "#884444",
    background_color = "#113311",
    widget = wibox.widget.graph,
    },
    id = "mylgc",
    widget = wibox.container.mirror,
    reflection = {horizontal = true, vertical = false},
  },
  {
    id           = "mylt",
    text         = "0",
    align        = "center",
    widget       = wibox.widget.textbox,
  },
  layout      = wibox.layout.stack,
  update_load = function(self, sload)
    self.mylgc.mylg:add_value(tonumber(sload), 0)
    self.mylt.text = sload
  end
}

function update_systemload()
    awful.spawn.easy_async_with_shell("cat /proc/loadavg | cut -d \" \" -f 1", function(sout, sin, exitr, exitc)
      mysystemload:update_load(sout:gsub("\n", ""))
    end)
end

local myhexclock = wibox.widget {
  {
    {
      id           = "myhc",
      text         = "time",
      align        = "center",
      widget       = wibox.widget.textbox,
    },
    id = "myhcc",
    widget = wibox.container.background,
    bg = "#662222",
  },
  layout      = wibox.layout.stack,
  update_hclock = function(self, time)
    local hex = string.format("#%02X%02X%02X", math.floor(time.hour / 23 * 255), math.floor(time.min / 60 * 255), math.floor(time.sec / 60 * 255))
    self.myhcc.myhc.text = " " .. hex .. " "
    self.myhcc.bg = hex
  end
}

function update_hclock()
  local date = os.date("*t")
  myhexclock:update_hclock(date)
end

function naughty.config.notify_callback(args)
  awful.spawn{ "mpv", "--no-vid", "/home/syrup/.local/share/resources/voy_hail.mp3" }
  return args
end

-- update_folders() -- Make async

-- gears.timer {
--   timeout   = 60,
--   autostart = true,
--   callback  = update_folders,
-- }

-- update_filesystem() -- Make async

-- gears.timer {
--   timeout   = 60,
--   autostart = true,
--   callback  = update_filesystem,
-- }

update_vpn()

gears.timer {
  timeout   = 60,
  autostart = true,
  callback  = update_vpn,
}


update_wifi()

gears.timer {
  timeout   = 15,
  autostart = true,
  callback  = update_wifi,
}

update_systemload()

gears.timer {
  timeout   = 15,
  autostart = true,
  callback  = update_systemload,
}

update_dsbattery() -- Make async

gears.timer {
  timeout   = 60,
  autostart = true,
  callback  = update_dsbattery,
}

update_batterystatus()

gears.timer {
  timeout   = 10,
  autostart = true,
  callback  = update_batterystatus,
}

-- update_hclock() -- Make async

-- gears.timer {
--   timeout   = 1,
--   autostart = true,
--   callback  = update_hclock,
-- }

-- Create a wibox for each screen and add it
local taglist_buttons = gears.table.join(
                    awful.button({ }, 1, function(t) t:view_only() end),
                    awful.button({ modkey }, 1, function(t)
                                              if client.focus then
                                                  client.focus:move_to_tag(t)
                                              end
                                          end),
                    awful.button({ }, 3, awful.tag.viewtoggle),
                    awful.button({ modkey }, 3, function(t)
                                              if client.focus then
                                                  client.focus:toggle_tag(t)
                                              end
                                          end),
                    awful.button({ }, 4, function(t) awful.tag.viewnext(t.screen) end),
                    awful.button({ }, 5, function(t) awful.tag.viewprev(t.screen) end)
                )

local tasklist_buttons = gears.table.join(
                     awful.button({ }, 1, function (c)
                                              if c == client.focus then
                                                  c.minimized = true
                                              else
                                                  -- Without this, the following
                                                  -- :isvisible() makes no sense
                                                  c.minimized = false
                                                  if not c:isvisible() and c.first_tag then
                                                      c.first_tag:view_only()
                                                  end
                                                  -- This will also un-minimize
                                                  -- the client, if needed
                                                  client.focus = c
                                                  c:raise()
                                              end
                                          end),
                     awful.button({ }, 3, client_menu_toggle_fn()),
                     awful.button({ }, 4, function ()
                                              awful.client.focus.byidx(1)
                                          end),
                     awful.button({ }, 5, function ()
                                              awful.client.focus.byidx(-1)
                                          end))

local function set_wallpaper(s)
    -- Wallpaper
    if beautiful.wallpaper then
        local wallpaper = beautiful.wallpaper
        -- If wallpaper is a function, call it with the screen
        if type(wallpaper) == "function" then
            wallpaper = wallpaper(s)
        end
        gears.wallpaper.maximized(wallpaper, s, true)
    end
end

-- Re-set wallpaper when a screen's geometry changes (e.g. different resolution)
screen.connect_signal("property::geometry", set_wallpaper)

awful.screen.connect_for_each_screen(function(s)
    -- Wallpaper
    set_wallpaper(s)

    -- Each screen has its own tag table.
    awful.tag({ "", "", "", "4", "5", "6", "7", "8", "9" }, s, awful.layout.layouts[1])

    -- Create a promptbox for each screen
    s.mypromptbox = awful.widget.prompt()
    -- Create an imagebox widget which will contain an icon indicating which layout we're using.
    -- We need one layoutbox per screen.
    s.mylayoutbox = awful.widget.layoutbox(s)
    s.mylayoutbox:buttons(gears.table.join(
                           awful.button({ }, 1, function () awful.layout.inc( 1) end),
                           awful.button({ }, 3, function () awful.layout.inc(-1) end),
                           awful.button({ }, 4, function () awful.layout.inc( 1) end),
                           awful.button({ }, 5, function () awful.layout.inc(-1) end)))
    -- Create a taglist widget
    s.mytaglist = awful.widget.taglist(s, awful.widget.taglist.filter.all, taglist_buttons)

    -- Create a tasklist widget
    s.mytasklist = awful.widget.tasklist(s, awful.widget.tasklist.filter.currenttags, tasklist_buttons)

    -- Create the wibox
    s.mywibox = awful.wibar({ position = "top", screen = s })

    -- Add widgets to the wibox
    s.mywibox:setup {
        layout = wibox.layout.align.horizontal,
        { -- Left widgets
            layout = wibox.layout.fixed.horizontal,
            mylauncher,
            s.mytaglist,
            s.mypromptbox,
        },
        s.mytasklist, -- Middle widget
        { -- Right widgets
            layout = wibox.layout.fixed.horizontal,
            wibox.widget.systray(),
	    -- myfolders,
	    -- myfilesystem,
	    -- myupdatestatus,
        myvpn,
	    mywifi,
	    mysystemload,
	    mydsbattery,
	    mybatterybar,
	    -- myhexclock,
        mytextclock,
        s.mylayoutbox,
        },
    }
end)
-- }}}

-- {{{ Mouse bindings
root.buttons(gears.table.join(
    awful.button({ }, 3, function () mymainmenu:toggle() end),
    awful.button({ }, 4, awful.tag.viewnext),
    awful.button({ }, 5, awful.tag.viewprev)
))
-- }}}

-- {{{ Key bindings
globalkeys = gears.table.join(
    awful.key({ modkey,           }, "s",      hotkeys_popup.show_help,
              {description="show help", group="awesome"}),
    awful.key({ modkey,           }, "Left",   awful.tag.viewprev,
              {description = "view previous", group = "tag"}),
    awful.key({ modkey,           }, "Right",  awful.tag.viewnext,
              {description = "view next", group = "tag"}),
    awful.key({ modkey,           }, "Escape", awful.tag.history.restore,
              {description = "go back", group = "tag"}),

    awful.key({ modkey,           }, "j",
        function ()
            awful.client.focus.byidx( 1)
        end,
        {description = "focus next by index", group = "client"}
    ),
    awful.key({ modkey,           }, "k",
        function ()
            awful.client.focus.byidx(-1)
        end,
        {description = "focus previous by index", group = "client"}
    ),
    awful.key({ modkey,           }, "w", function () mymainmenu:show() end,
              {description = "show main menu", group = "awesome"}),

    -- Layout manipulation
    awful.key({ modkey, "Shift"   }, "j", function () awful.client.swap.byidx(  1)    end,
              {description = "swap with next client by index", group = "client"}),
    awful.key({ modkey, "Shift"   }, "k", function () awful.client.swap.byidx( -1)    end,
              {description = "swap with previous client by index", group = "client"}),
    awful.key({ modkey, "Control" }, "j", function () awful.screen.focus_relative( 1) end,
              {description = "focus the next screen", group = "screen"}),
    awful.key({ modkey, "Control" }, "k", function () awful.screen.focus_relative(-1) end,
              {description = "focus the previous screen", group = "screen"}),
    awful.key({ modkey,           }, "u", awful.client.urgent.jumpto,
              {description = "jump to urgent client", group = "client"}),
    awful.key({ modkey,           }, "Tab",
        function ()
            awful.client.focus.history.previous()
            if client.focus then
                client.focus:raise()
            end
        end,
        {description = "go back", group = "client"}),

    -- Standard program
    awful.key({ modkey,           }, "Return", function () awful.spawn(terminal) end,
              {description = "open a terminal", group = "launcher"}),
    awful.key({ modkey,           }, "#", function () awful.spawn("emacsclient -c -a ''") end,
              {description = "open an emacs client and launch the server", group = "launcher"}),
    awful.key({ modkey, "Control" }, "r", awesome.restart,
              {description = "reload awesome", group = "awesome"}),
    awful.key({ modkey, "Shift"   }, "q", awesome.quit,
              {description = "quit awesome", group = "awesome"}),

    awful.key({ modkey,           }, "l",     function () awful.tag.incmwfact( 0.05)          end,
              {description = "increase master width factor", group = "layout"}),
    awful.key({ modkey,           }, "h",     function () awful.tag.incmwfact(-0.05)          end,
              {description = "decrease master width factor", group = "layout"}),
    awful.key({ modkey, "Shift"   }, "h",     function () awful.tag.incnmaster( 1, nil, true) end,
              {description = "increase the number of master clients", group = "layout"}),
    awful.key({ modkey, "Shift"   }, "l",     function () awful.tag.incnmaster(-1, nil, true) end,
              {description = "decrease the number of master clients", group = "layout"}),
    awful.key({ modkey, "Control" }, "h",     function () awful.tag.incncol( 1, nil, true)    end,
              {description = "increase the number of columns", group = "layout"}),
    awful.key({ modkey, "Control" }, "l",     function () awful.tag.incncol(-1, nil, true)    end,
              {description = "decrease the number of columns", group = "layout"}),
    awful.key({ modkey,           }, "space", function () awful.layout.inc( 1)                end,
              {description = "select next", group = "layout"}),
    awful.key({ modkey, "Shift"   }, "space", function () awful.layout.inc(-1)                end,
              {description = "select previous", group = "layout"}),

    awful.key({ modkey, "Control" }, "n",
              function ()
                  local c = awful.client.restore()
                  -- Focus restored client
                  if c then
                      client.focus = c
                      c:raise()
                  end
              end,
              {description = "restore minimized", group = "client"}),

    -- Prompt
    awful.key({ modkey },            "r",     function () awful.screen.focused().mypromptbox:run() end,
              {description = "run prompt", group = "launcher"}),

    awful.key({ modkey }, "x",
              function ()
                  awful.prompt.run {
                    prompt       = "Run Lua code: ",
                    textbox      = awful.screen.focused().mypromptbox.widget,
                    exe_callback = awful.util.eval,
                    history_path = awful.util.get_cache_dir() .. "/history_eval"
                  }
              end,
              {description = "lua execute prompt", group = "awesome"}),
    -- Menubar
    awful.key({ modkey }, "p", function() menubar.show() end,
              {description = "show the menubar", group = "launcher"})
)

clientkeys = gears.table.join(
    awful.key({ modkey,           }, "f",
        function (c)
            c.fullscreen = not c.fullscreen
            c:raise()
        end,
        {description = "toggle fullscreen", group = "client"}),
    awful.key({ modkey, "Shift"   }, "c",      function (c) c:kill()                         end,
              {description = "close", group = "client"}),
    awful.key({ modkey, "Control" }, "space",  awful.client.floating.toggle                     ,
              {description = "toggle floating", group = "client"}),
    awful.key({ modkey, "Control" }, "Return", function (c) c:swap(awful.client.getmaster()) end,
              {description = "move to master", group = "client"}),
    awful.key({ modkey,           }, "o",      function (c) c:move_to_screen()               end,
              {description = "move to screen", group = "client"}),
    awful.key({ modkey,           }, "t",      function (c) c.ontop = not c.ontop            end,
              {description = "toggle keep on top", group = "client"}),
    awful.key({ modkey,           }, "n",
        function (c)
            -- The client currently has the input focus, so it cannot be
            -- minimized, since minimized clients can't have the focus.
            c.minimized = true
        end ,
        {description = "minimize", group = "client"}),
    awful.key({ modkey,           }, "m",
        function (c)
            c.maximized = not c.maximized
            c:raise()
        end ,
        {description = "(un)maximize", group = "client"}),
    awful.key({ modkey, "Control" }, "m",
        function (c)
            c.maximized_vertical = not c.maximized_vertical
            c:raise()
        end ,
        {description = "(un)maximize vertically", group = "client"}),
    awful.key({ modkey, "Shift"   }, "m",
        function (c)
            c.maximized_horizontal = not c.maximized_horizontal
            c:raise()
        end ,
        {description = "(un)maximize horizontally", group = "client"})
)

-- Bind all key numbers to tags.
-- Be careful: we use keycodes to make it work on any keyboard layout.
-- This should map on the top row of your keyboard, usually 1 to 9.
for i = 1, 9 do
    globalkeys = gears.table.join(globalkeys,
        -- View tag only.
        awful.key({ modkey }, "#" .. i + 9,
                  function ()
                        local screen = awful.screen.focused()
                        local tag = screen.tags[i]
                        if tag then
                           tag:view_only()
                        end
                  end,
                  {description = "view tag #"..i, group = "tag"}),
        -- Toggle tag display.
        awful.key({ modkey, "Control" }, "#" .. i + 9,
                  function ()
                      local screen = awful.screen.focused()
                      local tag = screen.tags[i]
                      if tag then
                         awful.tag.viewtoggle(tag)
                      end
                  end,
                  {description = "toggle tag #" .. i, group = "tag"}),
        -- Move client to tag.
        awful.key({ modkey, "Shift" }, "#" .. i + 9,
                  function ()
                      if client.focus then
                          local tag = client.focus.screen.tags[i]
                          if tag then
                              client.focus:move_to_tag(tag)
                          end
                     end
                  end,
                  {description = "move focused client to tag #"..i, group = "tag"}),
        -- Toggle tag on focused client.
        awful.key({ modkey, "Control", "Shift" }, "#" .. i + 9,
                  function ()
                      if client.focus then
                          local tag = client.focus.screen.tags[i]
                          if tag then
                              client.focus:toggle_tag(tag)
                          end
                      end
                  end,
                  {description = "toggle focused client on tag #" .. i, group = "tag"})
    )
end

clientbuttons = gears.table.join(
    awful.button({ }, 1, function (c) client.focus = c; c:raise() end),
    awful.button({ modkey }, 1, awful.mouse.client.move),
    awful.button({ modkey }, 3, awful.mouse.client.resize))

-- Set keys
root.keys(globalkeys)
-- }}}

-- {{{ Rules
-- Rules to apply to new clients (through the "manage" signal).
awful.rules.rules = {
    -- All clients will match this rule.
    { rule = { },
      properties = { border_width = beautiful.border_width,
                     border_color = beautiful.border_normal,
                     focus = awful.client.focus.filter,
                     raise = true,
                     keys = clientkeys,
                     buttons = clientbuttons,
                     screen = awful.screen.preferred,
                     placement = awful.placement.no_overlap+awful.placement.no_offscreen
     }
    },

    -- Floating clients.
    { rule_any = {
        instance = {
          "DTA",  -- Firefox addon DownThemAll.
          "copyq",  -- Includes session name in class.
        },
        class = {
          "Arandr",
          "Gpick",
          "Kruler",
          "MessageWin",  -- kalarm.
          "Sxiv",
          "Wpa_gui",
          "pinentry",
          "veromix",
          "xtightvncviewer"},

        name = {
          "Event Tester",  -- xev.
        },
        role = {
          "AlarmWindow",  -- Thunderbird's calendar.
          "pop-up",       -- e.g. Google Chrome's (detached) Developer Tools.
        }
      }, properties = { floating = true }},

    -- Add titlebars to normal clients and dialogs
    { rule_any = {type = { "normal", "dialog" }
      }, properties = { titlebars_enabled = true }
    },

    -- Set Firefox to always map on the tag named "2" on screen 1.
    -- { rule = { class = "Firefox" },
    --   properties = { screen = 1, tag = "2" } },
}
-- }}}

-- {{{ Signals
-- Signal function to execute when a new client appears.
client.connect_signal("manage", function (c)
    -- Set the windows at the slave,
    -- i.e. put it at the end of others instead of setting it master.
    -- if not awesome.startup then awful.client.setslave(c) end

    if awesome.startup and
      not c.size_hints.user_position
      and not c.size_hints.program_position then
        -- Prevent clients from being unreachable after screen count changes.
        awful.placement.no_offscreen(c)
    end
end)

-- Add a titlebar if titlebars_enabled is set to true in the rules.
client.connect_signal("request::titlebars", function(c)
    -- buttons for the titlebar
    local buttons = gears.table.join(
        awful.button({ }, 1, function()
            client.focus = c
            c:raise()
            awful.mouse.client.move(c)
        end),
        awful.button({ }, 3, function()
            client.focus = c
            c:raise()
            awful.mouse.client.resize(c)
        end)
    )

    awful.titlebar(c, {position = "left"}) : setup {
        { -- Left
            awful.titlebar.widget.iconwidget(c),
            buttons = buttons,
            layout  = wibox.layout.fixed.vertical
        },
        { -- Middle
	  { -- Rotate title
            { -- Title
                align  = "center",
                widget = awful.titlebar.widget.titlewidget(c),
		layout = wibox.layout.horizontal
            },
	    widget = wibox.container.rotate,
	    direction = "east"
	  },
            buttons = buttons,
            layout  = wibox.layout.flex.vertical
        },
        { -- Right
	    awful.titlebar.widget.floatingbutton (c),
	    awful.titlebar.widget.stickybutton   (c),
            awful.titlebar.widget.ontopbutton    (c),
	    awful.titlebar.widget.minimizebutton(c),
            awful.titlebar.widget.maximizedbutton(c),
            awful.titlebar.widget.closebutton    (c),
            layout = wibox.layout.fixed.vertical
        },
        layout = wibox.layout.align.vertical
    }
end)

-- Enable sloppy focus, so that focus follows mouse.
client.connect_signal("mouse::enter", function(c)
    if awful.layout.get(c.screen) ~= awful.layout.suit.magnifier
        and awful.client.focus.filter(c) then
        client.focus = c
    end
end)

client.connect_signal("focus", function(c) c.border_color = beautiful.border_focus end)
client.connect_signal("unfocus", function(c) c.border_color = beautiful.border_normal end)
-- }}}
